@@ -3,7 +3,7 @@
  *
  * Used in conjunction with the libxlsxwriter library.
  *
- * Copyright 2014-2018, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
+ * Copyright 2014-2019, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
  *
  */
 
@@ -40,13 +40,13 @@ LXW_RB_GENERATE_CHARTSHEET_NAMES(lxw_chartsheet_names, lxw_chartsheet_name,
 STATIC int
 _worksheet_name_cmp(lxw_worksheet_name *name1, lxw_worksheet_name *name2)
 {
-    return strcmp(name1->name, name2->name);
+    return lxw_strcasecmp(name1->name, name2->name);
 }
 
 STATIC int
 _chartsheet_name_cmp(lxw_chartsheet_name *name1, lxw_chartsheet_name *name2)
 {
-    return strcmp(name1->name, name2->name);
+    return lxw_strcasecmp(name1->name, name2->name);
 }
 
 /*
@@ -208,6 +208,8 @@ lxw_workbook_free(lxw_workbook *workbook)
     lxw_sst_free(workbook->sst);
     free(workbook->options.tmpdir);
     free(workbook->ordered_charts);
+    free(workbook->vba_project);
+    free(workbook->vba_codename);
     free(workbook);
 }
 
@@ -884,10 +886,10 @@ _prepare_drawings(lxw_workbook *self)
 {
     lxw_sheet *sheet;
     lxw_worksheet *worksheet;
-    lxw_image_options *image_options;
-    uint16_t chart_ref_id = 0;
-    uint16_t image_ref_id = 0;
-    uint16_t drawing_id = 0;
+    lxw_object_properties *object_props;
+    uint32_t chart_ref_id = 0;
+    uint32_t image_ref_id = 0;
+    uint32_t drawing_id = 0;
     uint8_t is_chartsheet;
 
     STAILQ_FOREACH(sheet, self->sheets, list_pointers) {
@@ -900,36 +902,36 @@ _prepare_drawings(lxw_workbook *self)
             is_chartsheet = LXW_FALSE;
         }
 
-        if (STAILQ_EMPTY(worksheet->image_data)
+        if (STAILQ_EMPTY(worksheet->image_props)
             && STAILQ_EMPTY(worksheet->chart_data))
             continue;
 
         drawing_id++;
 
-        STAILQ_FOREACH(image_options, worksheet->chart_data, list_pointers) {
+        STAILQ_FOREACH(object_props, worksheet->chart_data, list_pointers) {
             chart_ref_id++;
             lxw_worksheet_prepare_chart(worksheet, chart_ref_id, drawing_id,
-                                        image_options, is_chartsheet);
-            if (image_options->chart)
-                STAILQ_INSERT_TAIL(self->ordered_charts, image_options->chart,
+                                        object_props, is_chartsheet);
+            if (object_props->chart)
+                STAILQ_INSERT_TAIL(self->ordered_charts, object_props->chart,
                                    ordered_list_pointers);
         }
 
-        STAILQ_FOREACH(image_options, worksheet->image_data, list_pointers) {
+        STAILQ_FOREACH(object_props, worksheet->image_props, list_pointers) {
 
-            if (image_options->image_type == LXW_IMAGE_PNG)
+            if (object_props->image_type == LXW_IMAGE_PNG)
                 self->has_png = LXW_TRUE;
 
-            if (image_options->image_type == LXW_IMAGE_JPEG)
+            if (object_props->image_type == LXW_IMAGE_JPEG)
                 self->has_jpeg = LXW_TRUE;
 
-            if (image_options->image_type == LXW_IMAGE_BMP)
+            if (object_props->image_type == LXW_IMAGE_BMP)
                 self->has_bmp = LXW_TRUE;
 
             image_ref_id++;
 
             lxw_worksheet_prepare_image(worksheet, image_ref_id, drawing_id,
-                                        image_options);
+                                        object_props);
         }
     }
 
@@ -1135,6 +1137,10 @@ _write_file_version(lxw_workbook *self)
     LXW_PUSH_ATTRIBUTES_STR("lowestEdited", "4");
     LXW_PUSH_ATTRIBUTES_STR("rupBuild", "4505");
 
+    if (self->vba_project)
+        LXW_PUSH_ATTRIBUTES_STR("codeName",
+                                "{37E998C4-C9E5-D4B9-71C8-EB1FF731991C}");
+
     lxw_xml_empty_tag(self->file, "fileVersion", &attributes);
 
     LXW_FREE_ATTRIBUTES();
@@ -1150,6 +1156,10 @@ _write_workbook_pr(lxw_workbook *self)
     struct xml_attribute *attribute;
 
     LXW_INIT_ATTRIBUTES();
+
+    if (self->vba_codename)
+        LXW_PUSH_ATTRIBUTES_STR("codeName", self->vba_codename);
+
     LXW_PUSH_ATTRIBUTES_STR("defaultThemeVersion", "124226");
 
     lxw_xml_empty_tag(self->file, "workbookPr", &attributes);
@@ -1470,6 +1480,7 @@ workbook_new_opt(const char *filename, lxw_workbook_options *options)
     if (options) {
         workbook->options.constant_memory = options->constant_memory;
         workbook->options.tmpdir = lxw_strdup(options->tmpdir);
+        workbook->options.use_zip64 = options->use_zip64;
     }
 
     return workbook;
@@ -1717,6 +1728,22 @@ workbook_close(lxw_workbook *self)
             worksheet->active = 1;
     }
 
+    /* Set workbook and worksheet VBA codenames if a macro has been added. */
+    if (self->vba_project) {
+        if (!self->vba_codename)
+            workbook_set_vba_name(self, "ThisWorkbook");
+
+        STAILQ_FOREACH(sheet, self->sheets, list_pointers) {
+            if (sheet->is_chartsheet)
+                continue;
+            else
+                worksheet = sheet->u.worksheet;
+
+            if (!worksheet->vba_codename)
+                worksheet_set_vba_name(worksheet, worksheet->name);
+        }
+    }
+
     /* Set the defined names for the worksheets such as Print Titles. */
     _prepare_defined_names(self);
 
@@ -1727,13 +1754,15 @@ workbook_close(lxw_workbook *self)
     _add_chart_cache_data(self);
 
     /* Create a packager object to assemble sub-elements into a zip file. */
-    packager = lxw_packager_new(self->filename, self->options.tmpdir);
+    packager = lxw_packager_new(self->filename,
+                                self->options.tmpdir,
+                                self->options.use_zip64);
 
     /* If the packager fails it is generally due to a zip permission error. */
     if (packager == NULL) {
         fprintf(stderr, "[ERROR] workbook_close(): "
                 "Error creating '%s'. "
-                "Error = %s\n", self->filename, strerror(errno));
+                "System error = %s\n", self->filename, strerror(errno));
 
         error = LXW_ERROR_CREATING_XLSX_FILE;
         goto mem_error;
@@ -1749,26 +1778,47 @@ workbook_close(lxw_workbook *self)
     if (error == LXW_ERROR_CREATING_TMPFILE) {
         fprintf(stderr, "[ERROR] workbook_close(): "
                 "Error creating tmpfile(s) to assemble '%s'. "
-                "Error = %s\n", self->filename, strerror(errno));
+                "System error = %s\n", self->filename, strerror(errno));
     }
 
-    /* If LXW_ERROR_ZIP_FILE_OPERATION then errno is set by zlib. */
+    /* If LXW_ERROR_ZIP_FILE_OPERATION then errno is set by zip. */
     if (error == LXW_ERROR_ZIP_FILE_OPERATION) {
         fprintf(stderr, "[ERROR] workbook_close(): "
-                "Zlib error while creating xlsx file '%s'. "
-                "Error = %s\n", self->filename, strerror(errno));
+                "Zip ZIP_ERRNO error while creating xlsx file '%s'. "
+                "System error = %s\n", self->filename, strerror(errno));
+    }
+
+    /* If LXW_ERROR_ZIP_PARAMETER_ERROR then errno is set by zip. */
+    if (error == LXW_ERROR_ZIP_PARAMETER_ERROR) {
+        fprintf(stderr, "[ERROR] workbook_close(): "
+                "Zip ZIP_PARAMERROR error while creating xlsx file '%s'. "
+                "System error = %s\n", self->filename, strerror(errno));
+    }
+
+    /* If LXW_ERROR_ZIP_BAD_ZIP_FILE then errno is set by zip. */
+    if (error == LXW_ERROR_ZIP_BAD_ZIP_FILE) {
+        fprintf(stderr, "[ERROR] workbook_close(): "
+                "Zip ZIP_BADZIPFILE error while creating xlsx file '%s'. "
+                "This may require the use_zip64 option for large files. "
+                "System error = %s\n", self->filename, strerror(errno));
+    }
+
+    /* If LXW_ERROR_ZIP_INTERNAL_ERROR then errno is set by zip. */
+    if (error == LXW_ERROR_ZIP_INTERNAL_ERROR) {
+        fprintf(stderr, "[ERROR] workbook_close(): "
+                "Zip ZIP_INTERNALERROR error while creating xlsx file '%s'. "
+                "System error = %s\n", self->filename, strerror(errno));
     }
 
     /* The next 2 error conditions don't set errno. */
     if (error == LXW_ERROR_ZIP_FILE_ADD) {
         fprintf(stderr, "[ERROR] workbook_close(): "
-                "Zlib error adding file to xlsx file '%s'.\n",
-                self->filename);
+                "Zip error adding file to xlsx file '%s'.\n", self->filename);
     }
 
     if (error == LXW_ERROR_ZIP_CLOSE) {
         fprintf(stderr, "[ERROR] workbook_close(): "
-                "Zlib error closing xlsx file '%s'.\n", self->filename);
+                "Zip error closing xlsx file '%s'.\n", self->filename);
     }
 
 mem_error:
@@ -2114,6 +2164,14 @@ workbook_validate_sheet_name(lxw_workbook *self, const char *sheetname)
     if (strpbrk(sheetname, "[]:*?/\\"))
         return LXW_ERROR_INVALID_SHEETNAME_CHARACTER;
 
+    /* Check that the worksheet doesn't start or end with an apostrophe. */
+    if (sheetname[0] == '\'' || sheetname[strlen(sheetname) - 1] == '\'')
+        return LXW_ERROR_SHEETNAME_START_END_APOSTROPHE;
+
+    /* Check that the worksheet name isn't the reserved work "History". */
+    if (lxw_strcasecmp(sheetname, "history") == 0)
+        return LXW_ERROR_SHEETNAME_RESERVED;
+
     /* Check if the worksheet name is already in use. */
     if (workbook_get_worksheet_by_name(self, sheetname))
         return LXW_ERROR_SHEETNAME_ALREADY_USED;
@@ -2124,3 +2182,48 @@ workbook_validate_sheet_name(lxw_workbook *self, const char *sheetname)
 
     return LXW_NO_ERROR;
 }
+
+/*
+ * Add a vbaProject binary to the Excel workbook.
+ */
+lxw_error
+workbook_add_vba_project(lxw_workbook *self, const char *filename)
+{
+    FILE *filehandle;
+
+    if (!filename) {
+        LXW_WARN("workbook_add_vba_project(): "
+                 "filename must be specified.");
+        return LXW_ERROR_NULL_PARAMETER_IGNORED;
+    }
+
+    /* Check that the vbaProject file exists and can be opened. */
+    filehandle = lxw_fopen(filename, "rb");
+    if (!filehandle) {
+        LXW_WARN_FORMAT1("workbook_add_vba_project(): "
+                         "file doesn't exist or can't be opened: %s.",
+                         filename);
+        return LXW_ERROR_PARAMETER_VALIDATION;
+    }
+    fclose(filehandle);
+
+    self->vba_project = lxw_strdup(filename);
+
+    return LXW_NO_ERROR;
+}
+
+/*
+ * Set the VBA name for the workbook.
+ */
+lxw_error
+workbook_set_vba_name(lxw_workbook *self, const char *name)
+{
+    if (!name) {
+        LXW_WARN("workbook_set_vba_name(): " "name must be specified.");
+        return LXW_ERROR_NULL_PARAMETER_IGNORED;
+    }
+
+    self->vba_codename = lxw_strdup(name);
+
+    return LXW_NO_ERROR;
+}
